#include<stdio.h>
#include<stdlib.h>
#include<math.h>
#include"dSFMT.h"

#define Number		1000
#define dx			0.01



void PDF_normal( double *data , int data_num ,double data_min , int imax , double *output){	//PDF
	int i ;
	
	for( i=0 ; i<data_num ; i++ ){
		output[(int)( ( data[i] - data_min )/dx) ] += 1.0/data_num;
	}
	for( i=0 ; i<imax ; i++){
		output[i] /= dx;
	}
}


void PDF_log( double *data , int data_num , double data_min , int imax   , double *output){			//PDF : log-scale
	int i ;
	double *width;
	width = (double *)calloc( imax , sizeof(double));
	
	for( i = 0 ; i< data_num ; i++){
		output[ (int)(( log10(data[i]) - log10(data_min) ) / dx)] += 1.0/data_num;
	}
	for(i = 0; i< imax ; i++){
		width[i] = data_min * ( pow(10 , (i+1)*dx ) - pow(10 , i*dx ) );
	}
	for(i = 0; i< imax ; i++){
		output[i] /= width[i];
	}
	free(width);
}

int main( int argc , char *argv[] ){
	dsfmt_t dsfmt1;
	
	int i,j,k;
	
	//Random number
	double alpha = atof(argv[1]);
	double rand_uniform , *rand_pow , *rand_log;
	rand_pow = (double *)calloc( Number , sizeof(double) );
	rand_log = (double *)calloc( Number , sizeof(double) );
	
	//PDF of Random number
	double min_rand,max_rand;
	int Imax_rand;
	double *F_rand;
	double *x_PDF;
	double sum_x,sum_y;
	

	//alpha
	double sum , sum1, sum2;
	double min_ml , max_ml , min_ls, max_ls;
	int Imax_ml, Imax_ls;
	double *alpha_ml ,*alpha_ls;
	alpha_ml = (double * )calloc( Number , sizeof(double) );
	alpha_ls = (double * )calloc( Number , sizeof(double) );
	double *F_ml , *F_ls;
	
	
	FILE *fp_ml,*fp_ls;
	fp_ml = fopen("Max_Likelihood.dat","w");
	fp_ls = fopen("Least_Square.dat" ,"w");
	
	
	for(i=0;i<=Number-1;i++){					//The number of trial
		
		dsfmt_init_gen_rand(&dsfmt1,i);			//Seed
		sum = 0.0;
		
		for(j=0;j<=Number-1;j++){						//Generate the number
			rand_uniform = dsfmt_genrand_close_open(&dsfmt1);
			rand_pow[j] = pow( rand_uniform , -1.0/alpha);
			
			//Maximum Likelihood
			rand_log[j] = log(rand_pow[j]);
			sum += rand_log[j];
			
			//PDF of random number
			min_rand = (j==0 || min_rand > rand_pow[j] )? rand_pow[j] : min_rand;
			max_rand = (j==0 || max_rand < rand_pow[j] )? rand_pow[j] : max_rand;
		}
		
		//Maximum Likelihood
		alpha_ml[i] = (double)Number/sum;
		min_ml = (i==0 || min_ml > alpha_ml[i] )? alpha_ml[i] : min_ml;
		max_ml = (i==0 || max_ml < alpha_ml[i] )? alpha_ml[i] : max_ml;
		
		
		//PDF of random number
		Imax_rand = (int)( ( log10(max_rand) - log10(min_rand) ) /dx ) + 1;
		F_rand = (double * )calloc( Imax_rand, sizeof(double));
		PDF_log( rand_pow , Number , min_rand , Imax_rand , F_rand );
		
		
		//Output the PDF
		x_PDF = (double * )calloc( Imax_rand, sizeof(double));
		sum_x = sum_y = 0.0;
		
		for(k=0;k<Imax_rand;k++){
			x_PDF[k] = min_rand * pow( 10 , (k+0.5)* dx );
			
			if( F_rand[k] != 0.0 ){
				sum_x += log(x_PDF[k]);
				sum_y += log(F_rand[k]);
			}
		}
		
		sum_x /= (double)Imax_rand;
		sum_y /= (double)Imax_rand;
		
		//Least Square
		sum1 = sum2 = 0.0;
		for(k=0;k<Imax_rand;k++){
			if( F_rand[k] != 0.0 ){
				sum1 += ( log(x_PDF[k]) - sum_x )*( log(F_rand[k]) - sum_y );
				sum2 += pow( log(x_PDF[k]) - sum_x , 2.0  );
			}
		}
		
		alpha_ls[i] =  -sum1/sum2;
		min_ls = (i==0 || min_ls > alpha_ls[i] )? alpha_ls[i] : min_ls;
		max_ls = (i==0 || max_ls < alpha_ls[i] )? alpha_ls[i] : max_ls;
		
		free(F_rand);
		free(x_PDF);
		
	}
	//PDF of alpha
	Imax_ml = (int)( ( max_ml - min_ml ) / dx ) + 1;
	F_ml = (double * )calloc( Imax_ml, sizeof(double));
	PDF_normal( alpha_ml , Number ,min_ml , Imax_ml  ,F_ml);
	for(k=0 ; k<Imax_ml ; k++){
		fprintf( fp_ml , "%f %f\n", min_ml+(k+0.5)*dx ,F_ml[k]);
	}
	
	Imax_ls = (int)( ( max_ls - min_ls ) / dx ) + 1;
	F_ls = (double * )calloc( Imax_ls, sizeof(double));
	PDF_normal( alpha_ls , Number ,min_ls , Imax_ls, F_ls);
	for(k=0 ; k<Imax_ls ; k++){
		fprintf( fp_ls , "%f %f\n", min_ls+(k+0.5)*dx ,F_ls[k]);
	}
	
	
	
	fclose(fp_ml);
	fclose(fp_ls);
	free(alpha_ml);
	free(alpha_ls);
	free(F_ml);
	free(F_ls);
	
	return 0;
}



